#title = "選択解除 (位置を復帰・非スクロール)"
#tooltip = "範囲選択を解除 (スクロールしない) または 位置を復帰"
#include "include/IO.js"
#icon = "selection_collapse.ico"
// #icon = "Mery用 マテリアルデザインっぽいアイコン.icl",212

// 推奨割当：「Esc」

/**
 * sukemaru, 2019/10/21 - 2019/11/13 【include 版】
 * 要：include ライブラリ
 * 
 * ・「すべて選択」マクロ実行前の
 *    選択範囲（キャレット位置）とスクロール位置の情報を
 *   JSON ファイルから読みこんで復元する（位置を復帰）。
 * ・「すべて選択」以外の範囲選択状態から実行したときは
 *    スクロールなしで範囲選択を解除する。
 * ・「範囲選択なし」の状態から実行したときはキャレット位置までスクロールする。
 * 
 * ※「すべて選択」の状態から「位置を復帰」したときのみ
 *    当該ファイルの位置情報を JSON から削除する。
 */

// ---------- ▼ 設定項目 ▼ ----------

// 位置情報の保存用ファイルのベース名
var jsonName = "位置情報を保存";

  // ※ ｢すべて選択｣ ｢選択解除｣ ｢位置を復帰｣ マクロで同一の名前を指定すること
  // 保存先は…		（e.g. "位置情報を保存" の場合）
  // ・ポータブル版: Mery\Macros\MacroSettings\位置情報を保存.json
  // ・インストール版: %AppData%\Mery\MacroSettings\位置情報を保存.json
  
// ---------- ▲ 設定項目 ▲ ----------

var d = editor.ActiveDocument,  s = d.selection;
var dText = d.Text,  dLength = dText.length;
var act = s.GetActivePos();
var sx = ScrollX,  sy = ScrollY;
Redraw = false;

// 「すべて選択」状態のとき
if ( s.Text.length == dLength ) {
  var dPath = d.FullName || "無題";

  var setting = { fileList: [] };
  setting = IO.Deserialize( setting, jsonName );	// JSON 読み込み

  var list = setting.fileList;
  var len = list.length;
  var infoExist = false;
  var fileInfo;

  for ( var i = 0; i < len; i ++ ) {

    // リストにファイル情報があれば
    if ( list[i][0] == dPath ) {
      infoExist = true;
      fileInfo = list[i];

      /**
       * list[i] = [ dPath, dLength, anc, act, sx, sy ]
       *  フルパス、文字数
       *  選択範囲の開始／終了位置（anc, act）
       *  スクロール位置（sx, sy）
       */
      // 文書の文字数が同じなら、選択範囲とスクロール位置を復元
      if ( fileInfo[1] == dLength ) {
        Restore( fileInfo[3], fileInfo[4], fileInfo[5], fileInfo[2] );
      }
      // 文書の文字数が異なるなら、選択解除 (スクロールしない)
      else {
        Restore( act, sx, sy );
      }

      // リストからファイル情報を削除
      list.splice( i, 1 );
      setting.fileList = list;
      IO.Serialize( setting, jsonName );	// JSON 書き込み
      break;
    }
  }
}

// 「すべて選択」以外の範囲選択状態のときは、選択解除 (スクロールしない)
if ( ! infoExist && s.Text ) {
  Restore( act, sx, sy );
}

// 「選択範囲なし」の状態のときは、キャレット位置までスクロールする
else if ( ! infoExist && s.IsEmpty ) {
  s.SelectLine();
  s.SetActivePos( act );
}

Redraw = true;


// 関数 Restore( activePos, scrollX, scrollY [, anchorPos] )
function Restore( act, sx, sy, anc ) {
  anc = anc >= 0 ? anc : act;
  editor.ActiveDocument.selection.SetActivePos( anc );
  editor.ActiveDocument.selection.SetActivePos( act, true );
  ScrollX = sx;  ScrollY = sy;
}