#title = "位置を復帰／保存"
#tooltip = "選択範囲（カーソル位置）を復帰 または 位置情報を保存"
#include "include/IO.js"
#icon = "arrows_swap_vertical[1].ico"
// #icon = "Mery用 マテリアルデザインっぽいアイコン.icl",224

/**
 * sukemaru, 2019/11/13 【include 版】
 * 要：include ライブラリ
 * 要: GetKeyState.exe
 * 
 * ・Ctrl キーを押しながらこのマクロを実行したときは「位置情報を保存」する。
 *  ※ ファイル名、文字数、選択範囲（キャレット位置）とスクロール位置などの
 *    情報を JSON ファイルに書き出す。
 * 
 * ・Ctrl キーを押さずにこのマクロを実行したときは「位置を復帰」する。
 *  ※ Ctrl キーを押しながらこのマクロを実行して保存した位置情報か、
 *   「すべて選択」マクロ実行前のキャレット位置（選択範囲）を
 *    JSON ファイルから読みこんで復元する。
 * 
 * ・JSON ファイルに保存された情報と文字数がことなるときは、
 *   保存された行番号（論理行）の行頭に復帰する。
 *  ※ 保存されたスクロール位置を復帰せず、キャレット位置までスクロールする。
 * 
 * ・「位置を復帰」の前後でキャレット位置（選択範囲）に変化がないとき、
 *   または JSON ファイルに位置情報がないときは、
 *   キャレット位置にスクロールするだけ。
 * 
 * ・「位置を復帰」の動作では JSON ファイルに保存された位置情報を削除しない。
 *  ※ Ctrl キーを押しながらこのマクロを実行したとき、
 *     または「すべて選択」マクロ実行時の「位置情報を保存」処理のさいに、
 *     開かれていないファイルの位置情報は JSON から削除される。
 * 
 * ・このマクロで保存した位置情報を、当該ファイルのタブを閉じずに削除したい場合は、
 *   「すべて選択」マクロと「選択解除」マクロをつづけて実行する。
 *  ※「すべて選択」の状態から「選択解除」マクロを実行したとき（位置の復帰動作）
 *     当該ファイルの位置情報は JSON から削除される。
 *  ※ このマクロ単独で「位置情報を保存／位置を復帰」の運用をする場合、
 *     JSON から位置情報を削除するには、当該ファイルのタブを閉じた状態で
 *     別のタブ上から「位置情報を保存」操作をするしかない。
 */

// ---------- ▼ 設定項目 ▼ ----------

// ■ 位置情報の保存用ファイルのベース名
var jsonName = "位置情報を保存";

  // ※ ｢すべて選択｣ ｢選択解除｣ ｢位置を復帰｣ マクロで同一の名前を指定すること
  // 保存先は…		（e.g. "位置情報を保存" の場合）
  // ・ポータブル版: Mery\Macros\MacroSettings\位置情報を保存.json
  // ・インストール版: %AppData%\Mery\MacroSettings\位置情報を保存.json

// ---------- ▲ 設定項目 ▲ ----------

var $ctrl = 0;
var getKeyState = editor.FullName.replace( /[^\\]+$/i , "" )
                + "Macros\\GetKeyState.exe";
var Fso = new ActiveXObject( "Scripting.FileSystemObject" );
if ( Fso.FileExists( getKeyState ) ) {
  var WshShell = new ActiveXObject( "WScript.Shell" );
  $ctrl = WshShell.Run( "\"" + getKeyState + "\" ctrl", 0, true );
}

var d = editor.ActiveDocument,  s = d.selection;
var dPath = d.FullName || "無題";
var dText = d.Text,  dLength = dText.length;
var act = s.GetActivePos(),  anc = s.GetAnchorPos();
var sx = ScrollX,  sy = ScrollY;

var setting = { fileList: [] };
setting = IO.Deserialize( setting, jsonName );	// JSON 読み込み
var infoExist = false;

var list = setting.fileList;
var len = list.length;

// 位置を復帰
if ( $ctrl == 0 ) {
  Redraw = false;

  for ( var i = 0, fileInfo; i < len; i ++ ) {
    fileInfo = list[i];
      /**
       * list[i] = [ dPath, dLength, anc, act, sx, sy, ay ]
       *  フルパス、文字数
       *  選択範囲の開始／終了位置（anc, act）
       *  スクロール位置（sx, sy）
       */

    // リストにファイル情報があれば
    if ( fileInfo[0] == dPath ) {
      infoExist = true;

      // 文書の文字数が同じなら、選択範囲（キャレット位置）を復元
      if ( fileInfo[1] == dLength ) {
        s.SetAnchorPos( fileInfo[2] );
        s.SetActivePos( fileInfo[3], true );
        ScrollX = fileInfo[4];
        ScrollY = fileInfo[5];
      }
      else {
        // 文書の文字数が異なるなら、行番号（行頭）に復帰
        s.SetActivePoint( mePosLogical, 1, fileInfo[6], false );
      }
      // 実行前と選択範囲が同じならスクロールする
      if ( s.GetActivePos() == act && s.GetAnchorPos() == anc ) {
        var yy = s.GetActivePointY( mePosView );
        ScrollY = yy;
        // var xx = s.GetActivePointX( mePosView );
        // ScrollX = xx;
      }
      break;
    }
  }

  // ファイル情報がないときは、キャレット位置までスクロールするだけ
  if ( ! infoExist  ) {
    s.SetActivePos( anc );
    s.SetActivePos( act, true );
  }

  Redraw = true;
}

// 位置情報を保存
else if ( $ctrl == 1 ) {
  /**
   *  フルパス、文字数
   *  選択範囲の開始／終了位置（anc, act）
   *  スクロール位置（sx, sy）、アクティブ行（ay）
   */
  var ay = s.GetActivePointY( mePosLogical );
  for ( var i = 0, path; i < list.length; i ++ ) {
    path = list[i][0];

    // リストにファイルの情報があれば書き換える
    if ( path == dPath ) {
      infoExist = true;
      list[i] = [ dPath, dLength, anc, act, sx, sy, ay ];
    }

    // リストに現在開かれていないファイルの情報があれば削除する
    else {
      var eCount = Editors.Count;
      var docus,  dCount,  filePath,  hit;
      for ( var ee = 0, hit = 0; ee < eCount; ee ++ ) {
        docus = Editors.Item( ee ).Documents;
        dCount = docus.Count;
        for ( var dd = 0; dd < dCount; dd ++ ) {
          filePath = docus.Item( dd ).FullName;
          if ( path == filePath ) { hit ++ }
        }
      }
      if ( ! hit ) { list.splice( i --, 1 ) }
    }
  }

  // リストにファイルの情報がなければ追加する
  if ( ! infoExist ) {
    list.push( [ dPath, dLength, anc, act, sx, sy, ay ] );
  }

  setting.fileList = list;
  IO.Serialize( setting, jsonName );	// JSON 書き込み
}