#title = "すべて選択 (位置を保存・非スクロール)"
#tooltip = "位置情報を保存してから すべて選択 (スクロールしない)"
#include "include/IO.js"
#icon = "select_all.ico"
// #icon = "Mery用 マテリアルデザインっぽいアイコン.icl",111

// 推奨割当：「Ctrl + A」

/**
 * sukemaru, 2019/10/21 - 2019/11/13 【include 版】
 * 要：include ライブラリ
 * 
 * ・｢すべて選択｣ 前の選択範囲（キャレット位置）とスクロール位置などの
 *   情報を JSON ファイルに書き出す。
 *   → ｢選択解除｣ ｢位置を復帰｣ マクロで、保存した位置情報に復元可。
 * ※ すべて選択状態から「選択解除」マクロを実行（位置を復帰）すると、
 *    当該ファイルの位置情報は JSON から削除される。
 * 
 * ・JSON にはファイルパスごとに位置情報を保存する。
 * ・JSON のリストに現在開かれていないファイルの情報があれば削除する。
 */

// ---------- ▼ 設定項目 ▼ ----------

// 位置情報の保存用ファイルのベース名
var jsonName = "位置情報を保存";

  // ※ ｢すべて選択｣ ｢選択解除｣ ｢位置を復帰｣ マクロで同一の名前を指定すること
  // 保存先は…		（e.g. "位置情報を保存" の場合）
  // ・ポータブル版: Mery\Macros\MacroSettings\位置情報を保存.json
  // ・インストール版: %AppData%\Mery\MacroSettings\位置情報を保存.json
  
// ---------- ▲ 設定項目 ▲ ----------

var d = editor.ActiveDocument,  s = d.selection;
var dText = d.Text,  dLength = dText.length;

if ( s.Text.length != dLength ) {
  /**
   *  フルパス、文字数
   *  選択範囲の開始／終了位置（anc, act）
   *  スクロール位置（sx, sy）、アクティブ行（ay）
   */
  var dPath = d.FullName || "無題";
  var anc = s.GetAnchorPos(),  act = s.GetActivePos();
  var sx = ScrollX,  sy = ScrollY;
  var ay = s.GetActivePointY( mePosLogical );

  // すべて選択 (スクロールしない)
  Redraw = false;
  s.SelectAll();
  ScrollX = sx;  ScrollY = sy;
  Redraw = true;

  var setting = { fileList: [] };
  setting = IO.Deserialize( setting, jsonName );	// JSON 読み込み
  var list = setting.fileList;
  var infoExist = false;

  for ( var i = 0, path; i < list.length; i ++ ) {
    path = list[i][0];

    // リストにファイルの情報があれば書き換える
    if ( path == dPath ) {
      infoExist = true;
      list[i] = [ dPath, dLength, anc, act, sx, sy, ay ];
    }
    // リストに現在開かれていないファイルの情報があれば削除する
    else {
      var eCount = Editors.Count;
      var docus,  dCount,  filePath,  hit;
      for ( var ee = 0, hit = 0; ee < eCount; ee ++ ) {
        docus = Editors.Item( ee ).Documents;
        dCount = docus.Count;
        for ( var dd = 0; dd < dCount; dd ++ ) {
          filePath = docus.Item( dd ).FullName;
          if ( path == filePath ) { hit ++ }
        }
      }
      if ( ! hit ) { list.splice( i --, 1 ) }
    }
  }

  // リストにファイルの情報がなければ追加する
  if ( ! infoExist ) {
    list.push( [ dPath, dLength, anc, act, sx, sy, ay ] );
  }

  setting.fileList = list;
  IO.Serialize( setting, jsonName );	// JSON 書き込み
}


/*
 * JSON ファイルの初期値
 * {"fileList":[]}
 */